
#include <qregexp.h>
#include "UmlCom.h"
#include "UmlOperation.h"
#include "UmlArtifact.h"
#include "UmlPackage.h"
#include "UserInput.h"
#include "UmlClassView.h"
#include "UmlClass.h"
#include "UmlDeploymentView.h"

void UmlClassView::genCxxTestCase() {
// Get the children classes
QVector<UmlItem> myChildren = children();

for(unsigned int i=0; i < myChildren.size(); i++){
	// did we find a class?
	if(myChildren[i]->kind() == aClass){
		// tell the class to generate the test case for itself
		myChildren[i]->genCxxTestCase(); 
	}
}


}

UmlClass* UmlClassView::getClass(QCString & who) {
  const QVector<UmlItem> ch = children();

  for (unsigned i = 0; i != ch.size(); i += 1)
    if ((ch[i]->kind() == aClass) &&
        (ch[i]->name() == who))
      return (UmlClass *) ch[i];

  return 0;
}

//Generate the test class members
void UmlClassView::genGlobalFixture() {
  int num = 0;
  QCString gFixtureName;
  QCString suff;
  
  //====================================================================
  // Get user Input
  //====================================================================
  // change checkbox text of dialog in UserInput::getUserInput()
  UserInput::get_instance()->bWorldFixtureCheckBox = true; 
  
  // show user input dialog
  if( ! UserInput::get_instance()->getUserInput() ){
  	return;
  }
  
  cv = UserInput::get_instance()->get_selectedClassView();
  dv = UserInput::get_instance()->get_selectedDeploymentView();
  
  if (!cv) cv = this;
  
  //====================================================================
  // do the real work
  //====================================================================
  bool bWorldFixture = UserInput::get_instance()->get_bGenFixture();
  // Globals fixture name
  if( bWorldFixture )
  	gFixtureName = "WorldFixture"; 
  else
  	gFixtureName = "GlobalFixture"; 
  
  // The property GlobalFixtureSuffix in the project's root package holds the number of
  // global fixture to be used for generating next fixture class in this view
  bool bIsNum = false;
  UmlPackage* proj = UmlBasePackage::getProject();
  
  if ( proj->propertyValue("GlobalFixtureSuffix", suff) ) {
  	num = suff.toInt(&bIsNum); // new fixture's number
  }else{
  	num = 1; // The first global fixture
  	suff = "1";
  }
  
  gFixtureName += suff; // add suffix
  
  UmlCom::trace("\nGenerating Global Fixture <b>" + gFixtureName + "</b>");
  
  // create fixture class
  UmlClass* pFixtureClass = cv->getClass(gFixtureName);  // maybe it already exists!
  if( !pFixtureClass ){ // if not found an existing class
  	pFixtureClass  = UmlClass::create(cv, gFixtureName);
  	pFixtureClass->set_CppDecl("${comment}${template}class ${name} : public CxxTest::GlobalFixture {\n${members}};\n\n${inlines}");
  	pFixtureClass->set_PropertyValue("CxxTestGlobalFixture", "1");
  }
  
  // Add CxxTest Fixture methods
  if(pFixtureClass) { // if new class was created
  	UmlTypeSpec retType;
  	retType.explicit_type = "bool";
  
  	UmlOperation* pOp;
  	bool bNewCreated = false;
  
  	// Setup global fixture methods
  	bNewCreated = UmlOperation::genTest(pFixtureClass, "setUp");
  	if(bNewCreated){
  		pOp = pFixtureClass->getOperation("setUp");
  		pOp->set_ReturnType( retType );
  		pOp->set_CppBody("TS_WARN(\"setUp method not implemented\");\nreturn true;\n" );
  	}
  
  	bNewCreated = UmlOperation::genTest(pFixtureClass, "tearDown");	
  	if(bNewCreated){		
  		pOp = pFixtureClass->getOperation("tearDown");
  		pOp->set_ReturnType( retType );
  		pOp->set_CppBody("TS_WARN(\"tearDown method not implemented\");\nreturn true;\n" );
  	}
  
  	// Setup world fixture methods
  	if( bWorldFixture ){
  		bNewCreated = UmlOperation::genTest(pFixtureClass, "setUpWorld");
  		if(bNewCreated){		
  			pOp = pFixtureClass->getOperation("setUpWorld");
  			pOp->set_ReturnType( retType );
  			pOp->set_CppBody("TS_WARN(\"setUpWorld method not implemented\");\nreturn true;\n" );
  		}
  
  		bNewCreated = UmlOperation::genTest(pFixtureClass, "tearDownWorld");			
  		if(bNewCreated){		
  			pOp = pFixtureClass->getOperation("tearDownWorld");
  			pOp->set_ReturnType( retType );
  			pOp->set_CppBody("TS_WARN(\"tearDownWorld method not implemented\");\nreturn true;\n" );
  		}
  	}
  }
  
  // Associate to the AllTests artifact
  if(dv){
  	UmlArtifact::genTestArtifact(dv, pFixtureClass);
  }
  
  // increment and store suffix for next time use
  num++;
  
  QCString strNum;
  strNum.setNum(num);
  
  proj->set_PropertyValue("GlobalFixtureSuffix", strNum );
  
}

