
#include "UmlOperation.h"
#include "UmlClass.h"

#include <ctype.h>
#include <qregexp.h>
#include "UmlCom.h"
bool UmlOperation::genTest(UmlClass * testClass) {
if(visibility() == PublicVisibility) {
	UmlCom::trace(QCString("generating test for public method ") + name() );

	QCString _name;
	QCString methodName;

	_name = name();
	// strip and replace white space with and ''
	_name.simplifyWhiteSpace();
	_name.replace(QRegExp("\\s"), "");

	// Check this method is an operator, and get name from operatorMap
	if ( operatorMap.find( _name ) != operatorMap.end() ){ // found?
		_name = operatorMap[_name];	// Get name from operator name map
	}else{
		// Check if this operation is a destructor
		if( _name[0] == '~' ) {// destructor
			UmlCom::trace(QCString("  ... skipping ") + _name );
			return false; // skip destructor
		}else{
			_name[0] = toupper(_name[0]);

		}
	}
	
	// Prefix the name with 'test'
	methodName = "test" + _name;

	// Create the new operation
	UmlTypeSpec retType;
	retType.explicit_type = "void";

	UmlOperation* newOp= testClass->getOperation(methodName);  // does this operation already exist?
	if( ! newOp  ){
		try{
			newOp = create(testClass, methodName);
			newOp->set_ReturnType(retType); // set return type of operation
			newOp->set_isCppInline(true);	// make it inline operation
			newOp->set_CppBody("\n\tTS_WARN(\"Test " + methodName + "() not implemented\");\n");
			return true; // new operation generated
		}catch(...){
			UmlCom::trace("<font color='red'>Error: could not generator test for operation <b>" + name() + "<b></font>");
			return false; // existing operation was found
		}
	}else{
		// Reset the existing operation's properties
		newOp->set_ReturnType(retType); // set return type of operation
		newOp->set_isCppInline(true);	// make it inline operation
		return false; // existing operation was found
	}

}

return false; // existing operation was found
}

bool UmlOperation::genTest(UmlClass * testClass, QCString name)
{
// Create the new operation
UmlOperation* newOp= testClass->getOperation(name); // we don't want two or more tests with same name

UmlTypeSpec retType;
retType.explicit_type = "void";

if( ! newOp  ){
	try{
		newOp = create(testClass, name);

		newOp->set_ReturnType(retType); // set return type of operation
		newOp->set_isCppInline(true);	// make it inline operation
		newOp->set_CppBody("\n\tTS_WARN(\"method " + name + "() not implemented\");\n");
		return true; // new test operation created
	}catch(...){
		UmlCom::trace("<font color='red'>Error: could not generator test for operation <b>" + name + "<b></font>");
	}
}else{
	// Reset the existing operation's properties
	newOp->set_ReturnType(retType); // set return type of operation
	newOp->set_isCppInline(true);	// make it inline operation
	return false;	// existing test operation with same name found
}

return false;	// existing test operation with same name found
}

