#ifndef _UMLOPERATION_H
#define _UMLOPERATION_H


#include <qmap.h>
#include "UmlBaseOperation.h"
#include <qcstring.h>

class UmlClass;

// This class manages 'operations', notes that the class 'UmlClassItem'
// is a mother class of all the class's children.
//
// You can modify it as you want (except the constructor)
class UmlOperation : public UmlBaseOperation {
  public:
    UmlOperation(void * id, const QCString & n);

    //Generate a test for an operation of a class
    //
    //Returns:
    // - true if a new test operation was generated.
    // - false if an existing test operation with the same name alread existed
    virtual bool genTest(UmlClass * testClass);

    //Generate a test by name
    //
    //Returns:
    // - true if a new test operation was generated.
    // - false if an existing test operation with the same name alread existed
    static bool genTest(UmlClass * testClass, QCString name);


  protected:
    QMap<QCString, QCString> operatorMap;

};

inline UmlOperation::UmlOperation(void * id, const QCString & n) : UmlBaseOperation(id, n)  {
  operatorMap["operator="] = QCString("OperatorAssignment");
  operatorMap["operator=="] = QCString("OperatorEqual");
  operatorMap["operator<"] = QCString("OperatorLess");
  operatorMap["operator>"] = QCString("OperatorGreater");
  operatorMap["operator<="] = QCString("OperatorLessOrEqual");
  operatorMap["operator>="] = QCString("OperatorGreaterOrEqual");
  operatorMap["operator!="] = QCString("OperatorNotEqual");
  operatorMap["operator+"] = QCString("OperatorPlus");
  operatorMap["operator-"] = QCString("OperatorMinus");
  operatorMap["operator*"] = QCString("OperatorMultiply");
  operatorMap["operator/"] = QCString("OperatorDivide");
  operatorMap["operator%"] = QCString("OperatorMod");
  operatorMap["operator+="] = QCString("OperatorPlusAssign");
  operatorMap["operator-="] = QCString("OperatorMinusAssign");
  operatorMap["operator*="] = QCString("OperatorMulAssign");
  operatorMap["operator/="] = QCString("OperatorDivAssign");
  operatorMap["operator%="] = QCString("OperatorModAssign");
  operatorMap["operator>>"] = QCString("OperatorRightShift");
  operatorMap["operator<<"] = QCString("OperatorLeftShift");
  operatorMap["operator>>="] = QCString("OperatorRightShiftAssign");
  operatorMap["operator<<="] = QCString("OperatorLeftShiftAssign");
  operatorMap["operator()"] = QCString("OperatorCall");
  operatorMap["operator[]"] = QCString("OperatorSubscript");
  operatorMap["operator^"] = QCString("OperatorPower");
  operatorMap["operator^="] = QCString("OperatorPowerAssign");
  operatorMap["operator&"] = QCString("OperatorBitwiseAnd");
  operatorMap["operator&="] = QCString("OperatorBitwiseAndAssign");
  operatorMap["operator|"] = QCString("OperatorBitwiseOr");
  operatorMap["operator|="] = QCString("OperatorBitwiseOrAssign");
  operatorMap["operator~"] = QCString("OperatorBitwiseNot");
  operatorMap["operator~="] = QCString("OperatorBitwiseNotAssign");
  operatorMap["operator&&"] = QCString("OperatorLogicalAnd");
  operatorMap["operator&&="] = QCString("OperatorLogicalAndAssign");
  operatorMap["operator||"] = QCString("OperatorLogicalOr");
  operatorMap["operator||="] = QCString("OperatorLogicalOrAssign");
  operatorMap["operator++"] = QCString("OperatorIncrement");
  operatorMap["operator--"] = QCString("OperatorDecrement");
  operatorMap["operator->"] = QCString("OperatorMemberAccessViaPtr");
  operatorMap["operator->*"] = QCString("OperatorPtrMemberAccessViaPtr");
  operatorMap["operatornew"] = QCString("OperatorNew");
  operatorMap["operatornew[]"] = QCString("OperatorNewArray");
  operatorMap["operatordelete"] = QCString("OperatorDelete");
  operatorMap["operatordelete[]"] = QCString("OperatorDeleteArray");
}

#endif
